﻿package  {
	
	import flash.display.Sprite
	import flash.display.Bitmap
	import flash.events.Event
	//import flash.utils.setTimeout;
	
	/**
	* BitmapClipSource로 무비 클립을 찍은 후 BitmapClip로 화면에 표시합니다.
	* 따라서 한 소스를 여러 BitmapClip 객체에서 사용할 수 있습니다.
	* <br/> TODO <br/>
	* 무비클립에 그림 - 자식 무비클립 - 그림 순으로 그림 사이에 무비 클립이 있으면 깊이 정렬이 어긋납니다. <br/>
	* 동일한 자식 무비 클립이 여러 프레임에 걸쳐 나올 때 하나의 자식으로 처리하지 못합니다. <br/>
	* play(), gotoAndPlay() 호출 시 처음에는 다음 프레임으로 넘어가는 시간 간격이 틀립니다. <br/>
	* 모션 트위닝을 인식하지 못합니다. <br/>
	* 그 외의 문제를 제보해주세요.
	*/
	public class BitmapClip extends Sprite {
		
		private var _src:BitmapClipSource
		private var _frame:int
		private var _bitmap:Bitmap
		private var _children:Vector.<Object> = new Vector.<Object>()
		
		public function BitmapClip(src:BitmapClipSource,
								   					pixelSnapping:String="auto", smoothing:Boolean=true) {
			_src = src
			addChild(_bitmap = new Bitmap(null, pixelSnapping, smoothing))
			
			var childSrc:BitmapClipSource
			var len:int = _src._children.length
			for(var i:int=0 ; i<len ; i++){
				_children[i] = {}
				for(var childSrcName:String in _src._children[i]){
					childSrc = _src._children[i][childSrcName]
					_children[i][childSrcName] = new BitmapClip(childSrc, pixelSnapping, smoothing)
					_children[i][childSrcName].x = childSrc.x
					_children[i][childSrcName].y = childSrc.y
				}
			}
			gotoAndStop(1)
			play()
		}
		
		/**
		* 자식에 접근합니다. bdmc.child("aa") 는 mc.aa에 해당합니다.
		*/
		public function child(name:String):BitmapClip {
			return _children[_frame-1][name]
		}
		
		private function goto(frame:int):void {
			if(_frame == frame) return
			if(_frame) for each(var child:BitmapClip in _children[_frame-1]){
				child.stop()
				removeChild(child)
			}
			
			_frame = frame
			if(_src._captures[_frame-1]){
				_bitmap.bitmapData = _src._captures[_frame-1]
				_bitmap.x = _src._offsets[_frame-1].x
				_bitmap.y = _src._offsets[_frame-1].y
				_bitmap.visible = true
				for each(child in _children[_frame-1]){
					addChild(child)
					child.gotoAndPlay(1)
				}
			} else _bitmap.visible = false
		}
		
		/**
		* 무비 클립과 동일합니다.
		*/
		public function gotoAndStop(frame:int):void {
			goto(frame)
			stop()
		}
		
		/**
		* 무비 클립과 동일합니다.
		*/
		public function prevFrame():void {
			if(_frame != 1){
				gotoAndStop(_frame-1)
			}
		}
		/**
		* 무비 클립과 동일합니다.
		*/
		public function nextFrame():void {
			if(_frame != _src._captures.length){
				gotoAndStop(_frame+1)
			}
		}
		
		/**
		* 무비 클립과 동일합니다.
		*/
		public function get currentFrame():int { return _frame }
		/**
		* 무비 클립과 동일합니다.
		*/
		public function get totalFrames():int { return _src._captures.length }
		
		/**
		* 무비 클립과 동일합니다.
		*/
		public function play():void {
			addEventListener("enterFrame", apeMcPlay)
			//flash.utils.setTimeout(addEventListener, 0, "enterFrame", apeMcPlay)
		}
		/**
		* 무비 클립과 동일합니다.
		*/
		public function stop():void {
			removeEventListener("enterFrame", apeMcPlay)
		}
		/**
		* 무비 클립과 동일합니다.
		*/
		public function gotoAndPlay(frame:int):void {
			goto(frame)
			play()
		}
		private function apeMcPlay(e:Event):void {
			if(currentFrame == totalFrames) goto(1)
			else goto(_frame+1)
		}
		
		/**
		* 생성자에서 인자로 받은 BitmapClipSource 객체입니다.
		*/
		public function get source():BitmapClipSource { return _src }
		public function set source(v:BitmapClipSource):void { _src = v }

	}
	
}
