﻿package  {
	
	import flash.display.DisplayObject
	import flash.display.BitmapData
	import flash.display.MovieClip
	import flash.display.Bitmap
	import flash.display.Sprite
	import flash.geom.Point
	import flash.geom.Matrix
	import flash.geom.Rectangle
	
	/**
	* 무비클립을 분석하여 각 프레임을 개별 비트맵데이터에 저장합니다.
	* 단, 이미지를 올바르게 저장하려면 인자로 넘긴 무비클립이 임의 컨테이너의 자식이여야 합니다.
	* 무비클립의 부모가 없다면 임시로 컨테이너에 추가하였다 제거하는 작업이 일어나니
	* added, removed 이벤트 수신자가 등록된 무비클립이라면 주의하시기 바랍니다.
	* 저장한 이미지를 화면에 표시하려면 BitmapClip 클래스를 사용합니다.
	*/
	public class BitmapClipSource {
		
		private var temp_container:Sprite = new Sprite
		
		internal var _captures:Vector.<BitmapData> = new <BitmapData>[]
		internal var _children:Vector.<Object> = new <Object>[]
		internal var _offsets:Vector.<Point> = new <Point>[]
		internal var _x:Number=0, _y:Number=0
		
		/**
		* 생성자.
		*/
		public function BitmapClipSource(mc:MovieClip) {
			var x0:Number = mc.x
			var y0:Number = mc.y
			
			if(mc.parent == null) temp_container.addChild(mc)
			
			mc.x = mc.y = 0
			for(var i:int=0 ; i<mc.totalFrames ; i++) capture(mc, i)
			mc.x = x0
			mc.y = y0
			
			if(temp_container.contains(mc)){
				temp_container.removeChild(mc)
			}
			temp_container = null
		}
		
		// i : 찍을 프레임
		private function capture(mc:MovieClip, i:uint):void {
			var child:DisplayObject, child_mc:MovieClip, bd:BitmapData
			var R:Rectangle, M:Matrix
			
			mc.gotoAndStop(i+1)
			
			if(mc.width == 0 && mc.height == 0){
				_captures.push(null)
				_offsets.push(null)
				_children.push(null)
			} else {
				M = mc.transform.matrix
				R = mc.getBounds(mc.parent)
				bd = new BitmapData(mc.width+0.5, mc.height+0.5, true, 0)
				_captures.push(bd)
				_offsets.push(new Point(R.x, R.y))
				_children[i] = {}
				for(var j:int=0 ; j<mc.numChildren ; j++){
					child = mc.getChildAt(j)
					if(child is MovieClip){
						child_mc = child as MovieClip
						_children[i][child_mc.name] = new BitmapClipSource(child_mc)
						_children[i][child_mc.name]._x = child.x
						_children[i][child_mc.name]._y = child.y
					}else{
						M.tx = child.x - R.x
						M.ty = child.y - R.y
						bd.draw(child, M, child.transform.colorTransform)
					}
				}
			}
		}
		
		internal function get x():Number { return _x }
		internal function get y():Number { return _y }
		
		internal function child(frame:int, name:String):BitmapClipSource {
			return _children[frame-1][name]
		}
		
		/**
		* 내부의 비트맵데이터 자료를 처분합니다.
		*/
		public function dispose():void {
			for each(var bd:BitmapData in _captures) bd.dispose()
			_captures.length = 0
			_children.length = 0
			_offsets.length = 0
		}
		
		/**
		* 디버깅용
		*/
		public function toString():String {
			var str:String = ""
			for(var i:int=0 ; i<_captures.length ; i++){
				str += "frame" + (i+1) + "\n"
				for each(var ch:BitmapClipSource in _children[i]){
					str += "\t" + ch.toString()
				}
			}
			return str
		}

	}
	
}
