﻿package  {
	
	import flash.display.DisplayObject
	import flash.display.BitmapData
	import flash.display.MovieClip
	import flash.display.Bitmap
	import flash.display.Sprite
	import flash.geom.Point
	import flash.geom.Matrix
	import flash.geom.Rectangle
	
	/**
	* 무비클립을 분석하여 각 프레임을 개별 비트맵데이터에 저장합니다.
	* 단, 이미지를 올바르게 저장하려면 인자로 넘긴 무비클립이 임의 컨테이너의 자식이여야 합니다.
	* 무비클립의 부모가 없다면 임시로 컨테이너에 추가하였다 제거하는 작업이 일어나니
	* added, removed 이벤트 수신자가 등록된 무비클립이라면 주의하시기 바랍니다.
	* 저장한 이미지를 화면에 표시하려면 BitmapSprite 클래스를 사용합니다.
	*/
	public class BitmapSpriteSource {
		
		private var temp_container:Sprite = new Sprite
		
		internal var _captures:Vector.<BitmapData> = new <BitmapData>[]
		internal var _offsets:Vector.<Point> = new <Point>[]
		internal var _x:Number=0, _y:Number=0
		
		/**
		* 생성자.
		*/
		public function BitmapSpriteSource(mc:MovieClip) {
			var x0:Number = mc.x
			var y0:Number = mc.y
			
			if(mc.parent == null) temp_container.addChild(mc)
			
			mc.x = mc.y = 0
			for(var i:int=0 ; i<mc.totalFrames ; i++) capture(mc, i)
			mc.x = x0
			mc.y = y0
			
			if(temp_container.contains(mc)){
				temp_container.removeChild(mc)
			}
			temp_container = null
		}
		
		// i : 찍을 프레임
		private function capture(mc:MovieClip, i:uint):void {
			var bd:BitmapData
			var R:Rectangle, M:Matrix
			
			mc.gotoAndStop(i+1)
			if(mc.width == 0 && mc.height == 0){
				_captures.push(null)
				_offsets.push(null)
			} else {
				M = mc.transform.matrix
				R = mc.getBounds(mc.parent)
				bd = new BitmapData(mc.width+0.5, mc.height+0.5, true, 0)
				M.tx = mc.x - R.x
				M.ty = mc.y - R.y
				bd.draw(mc, M, mc.transform.colorTransform)
				_captures.push(bd)
				_offsets.push(new Point(R.x, R.y))
			}
		}
		
		internal function get x():Number { return _x }
		internal function get y():Number { return _y }
		
		/**
		* 내부의 비트맵데이터 자료를 처분합니다.
		*/
		public function dispose():void {
			for each(var bd:BitmapData in _captures) bd.dispose()
			_captures.length = 0
			_offsets.length = 0
		}
		
		/**
		* 각 프레임이 찍힌 비트맵데이터들을 반환합니다.
		*/
		public function get captures():Vector.<BitmapData> {
			return _captures
		}
		/**
		* 각 프레임에서 무비클립의 기준점으로부터 무비클립 경계의 좌상단 모서리로 향하는 벡터들의 모음입니다.
		*/
		public function get offsets():Vector.<Point> {
			return _offsets
		}

	}
	
}
